@@ -115,8 +115,8 @@
 //! [container types]: https://dbus.freedesktop.org/doc/dbus-specification.html#container-types
 //! [slice]: https://doc.rust-lang.org/std/primitive.slice.html
 //! [`Vec`]: https://doc.rust-lang.org/std/vec/struct.Vec.html
-//! [`arrayvec::ArrayVec`]: https://docs.rs/arrayvec/0.5.1/arrayvec/struct.ArrayVec.html
-//! [`arrayvec::ArrayString`]: https://docs.rs/arrayvec/0.5.1/arrayvec/struct.ArrayString.html
+//! [`arrayvec::ArrayVec`]: https://docs.rs/arrayvec/0.7.1/arrayvec/struct.ArrayVec.html
+//! [`arrayvec::ArrayString`]: https://docs.rs/arrayvec/0.7.1/arrayvec/struct.ArrayString.html
 //! [`Value` module documentation]: enum.Value.html
 
 #[macro_use]
@@ -165,6 +165,9 @@ mod maybe;
 #[cfg(feature = "gvariant")]
 pub use crate::maybe::*;
 
+mod optional;
+pub use crate::optional::*;
+
 mod value;
 pub use value::*;
 
@@ -196,10 +199,7 @@ mod framing_offset_size;
 mod framing_offsets;
 mod signature_parser;
 
-// FIXME: Re-export derive macros from the crate root with the next breaking-change release.
-pub mod derive {
-    pub use zvariant_derive::{DeserializeDict, OwnedValue, SerializeDict, Type, TypeDict, Value};
-}
+pub use zvariant_derive::{DeserializeDict, OwnedValue, SerializeDict, Type, TypeDict, Value};
 
 // Required for the macros to function within this crate.
 extern crate self as zvariant;
@@ -228,16 +228,15 @@ mod tests {
     use glib::{Bytes, FromVariant, Variant};
     use serde::{Deserialize, Serialize};
 
-    use zvariant_derive::{DeserializeDict, SerializeDict, Type, TypeDict};
-
     use crate::{
         from_slice, from_slice_fds, from_slice_for_signature, to_bytes, to_bytes_fds,
         to_bytes_for_signature,
     };
 
     use crate::{
-        Array, Basic, DeserializeValue, Dict, EncodingContext as Context, EncodingFormat, Error,
-        Fd, ObjectPath, Result, SerializeValue, Signature, Str, Structure, Type, Value,
+        Array, Basic, DeserializeDict, DeserializeValue, Dict, EncodingContext as Context,
+        EncodingFormat, Error, Fd, ObjectPath, Result, SerializeDict, SerializeValue, Signature,
+        Str, Structure, Type, TypeDict, Value,
     };
 
     // Test through both generic and specific API (wrt byte order)
@@ -548,11 +547,11 @@ mod tests {
     #[cfg(feature = "arrayvec")]
     #[test]
     fn array_string_value() {
-        let s = ArrayString::<[_; 32]>::from_str("hello world!").unwrap();
+        let s = ArrayString::<32>::from_str("hello world!").unwrap();
         let ctxt = Context::<LE>::new_dbus(0);
         let encoded = to_bytes(ctxt, &s).unwrap();
         assert_eq!(encoded.len(), 17);
-        let decoded: ArrayString<[_; 32]> = from_slice(&encoded, ctxt).unwrap();
+        let decoded: ArrayString<32> = from_slice(&encoded, ctxt).unwrap();
         assert_eq!(&decoded, "hello world!");
     }
 
@@ -637,6 +636,7 @@ mod tests {
         // Array of u8
         //
         // First a normal Rust array that is actually serialized as a struct (thank you Serde!)
+        assert_eq!(<[u8; 2]>::signature(), "(yy)");
         let ay = [77u8, 88];
         let ctxt = Context::<LE>::new_dbus(0);
         let encoded = to_bytes(ctxt, &ay).unwrap();
@@ -654,7 +654,7 @@ mod tests {
         assert_eq!(encoded.len(), 6);
 
         #[cfg(feature = "arrayvec")]
-        let decoded: ArrayVec<[u8; 2]> = from_slice(&encoded, ctxt).unwrap();
+        let decoded: ArrayVec<u8, 2> = from_slice(&encoded, ctxt).unwrap();
         #[cfg(not(feature = "arrayvec"))]
         let decoded: Vec<u8> = from_slice(&encoded, ctxt).unwrap();
         assert_eq!(&decoded.as_slice(), &[77u8, 88]);
@@ -1014,6 +1014,23 @@ mod tests {
         }
     }
 
+    #[test]
+    fn struct_byte_array() {
+        let ctxt = Context::<LE>::new_dbus(0);
+        let value: (Vec<u8>, HashMap<String, Value<'_>>) = (Vec::new(), HashMap::new());
+        let value = zvariant::to_bytes(ctxt, &value).unwrap();
+        #[cfg(feature = "serde_bytes")]
+        let (bytes, map): (&serde_bytes::Bytes, HashMap<&str, Value<'_>>) =
+            zvariant::from_slice(&value, ctxt)
+                .expect("Could not deserialize serde_bytes::Bytes in struct.");
+        #[cfg(not(feature = "serde_bytes"))]
+        let (bytes, map): (&[u8], HashMap<&str, Value<'_>>) =
+            zvariant::from_slice(&value, ctxt).expect("Could not deserialize u8 slice in struct");
+
+        assert!(bytes.is_empty());
+        assert!(map.is_empty());
+    }
+
     #[test]
     fn struct_value() {
         // Struct->Value
@@ -1382,16 +1399,18 @@ mod tests {
         struct AStruct<'s> {
             field1: u16,
             field2: &'s [u8],
-            field3: i64,
+            field3: &'s [u8],
+            field4: i64,
         }
-        assert_eq!(AStruct::signature(), "(qayx)");
+        assert_eq!(AStruct::signature(), "(qayayx)");
         let s = AStruct {
             field1: 0xFF_FF,
             field2: &[77u8; 8],
-            field3: 0xFF_FF_FF_FF_FF_FF,
+            field3: &[77u8; 8],
+            field4: 0xFF_FF_FF_FF_FF_FF,
         };
         let encoded = to_bytes(ctxt, &s).unwrap();
-        assert_eq!(encoded.len(), 24);
+        assert_eq!(encoded.len(), 40);
         let decoded: AStruct<'_> = from_slice(&encoded, ctxt).unwrap();
         assert_eq!(decoded, s);
     }
